@@ -1,5 +1,7 @@
 /*
- *  Copyright (C) 2002 - 2005 Tomasz Kojm <tkojm@clamav.net>
+ *  Copyright (C) 2007-2009 Sourcefire, Inc.
+ *
+ *  Authors: Tomasz Kojm
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -28,10 +30,11 @@
 #include <sys/stat.h>
 #include <signal.h>
 #include <pthread.h>
+#include <string.h>
 
 #include "libclamav/clamav.h"
 
-#include "shared/cfgparser.h"
+#include "shared/optparser.h"
 #include "shared/output.h"
 
 #include "server.h"
@@ -40,7 +43,7 @@
 #include "clamuko.h"
 
 struct dazuko_access *acc;
-
+short int clamuko_scanning;
 static void clamuko_exit(int sig)
 {
 
@@ -67,7 +70,7 @@ void *clamukoth(void *arg)
 	const char *virname;
         struct sigaction act;
 	unsigned long mask = 0;
-	const struct cfgstruct *pt;
+	const struct optstruct *pt;
 	short int scan;
 	int sizelimit = 0;
 	struct stat sb;
@@ -87,6 +90,7 @@ void *clamukoth(void *arg)
     sigdelset(&sigset, SIGBUS);
 #endif
     pthread_sigmask(SIG_SETMASK, &sigset, NULL);
+    memset(&act, 0, sizeof(struct sigaction));
     act.sa_handler = clamuko_exit;
     sigfillset(&(act.sa_mask));
     sigaction(SIGUSR1, &act, NULL);
@@ -100,15 +104,15 @@ void *clamukoth(void *arg)
 	logg("Clamuko: Correctly registered with Dazuko.\n");
 
     /* access mask */
-    if(cfgopt(tharg->copt, "ClamukoScanOnOpen")->enabled) {
+    if(optget(tharg->opts, "ClamukoScanOnOpen")->enabled) {
 	logg("Clamuko: Scan-on-open mode activated.\n");
 	mask |= DAZUKO_ON_OPEN;
     }
-    if(cfgopt(tharg->copt, "ClamukoScanOnClose")->enabled) {
+    if(optget(tharg->opts, "ClamukoScanOnClose")->enabled) {
 	logg("Clamuko: Scan-on-close mode activated.\n");
 	mask |= DAZUKO_ON_CLOSE;
     }
-    if(cfgopt(tharg->copt, "ClamukoScanOnExec")->enabled) {
+    if(optget(tharg->opts, "ClamukoScanOnExec")->enabled) {
 	logg("Clamuko: Scan-on-exec mode activated.\n");
 	mask |= DAZUKO_ON_EXEC;
     }
@@ -125,7 +129,7 @@ void *clamukoth(void *arg)
 	return NULL;
     }
 
-    if((pt = cfgopt(tharg->copt, "ClamukoIncludePath"))->enabled) {
+    if((pt = optget(tharg->opts, "ClamukoIncludePath"))->enabled) {
 	while(pt) {
 	    if((dazukoAddIncludePath(pt->strarg))) {
 		logg("!Clamuko: Dazuko -> Can't include path %s\n", pt->strarg);
@@ -134,7 +138,7 @@ void *clamukoth(void *arg)
 	    } else
 		logg("Clamuko: Included path %s\n", pt->strarg);
 
-	    pt = (struct cfgstruct *) pt->nextarg;
+	    pt = (struct optstruct *) pt->nextarg;
 	}
     } else {
 	logg("!Clamuko: please include at least one path.\n");
@@ -142,7 +146,7 @@ void *clamukoth(void *arg)
 	return NULL;
     }
 
-    if((pt = cfgopt(tharg->copt, "ClamukoExcludePath"))->enabled) {
+    if((pt = optget(tharg->opts, "ClamukoExcludePath"))->enabled) {
 	while(pt) {
 	    if((dazukoAddExcludePath(pt->strarg))) {
 		logg("!Clamuko: Dazuko -> Can't exclude path %s\n", pt->strarg);
@@ -151,11 +155,11 @@ void *clamukoth(void *arg)
 	    } else
 		logg("Clamuko: Excluded path %s\n", pt->strarg);
 
-	    pt = (struct cfgstruct *) pt->nextarg;
+	    pt = (struct optstruct *) pt->nextarg;
 	}
     }
 
-    sizelimit = cfgopt(tharg->copt, "ClamukoMaxFileSize")->numarg;
+    sizelimit = optget(tharg->opts, "ClamukoMaxFileSize")->numarg;
     if(sizelimit)
 	logg("Clamuko: Max file size limited to %d bytes.\n", sizelimit);
     else
@@ -175,9 +179,9 @@ void *clamukoth(void *arg)
 		}
 	    }
 
-	    if(scan && cl_scanfile(acc->filename, &virname, NULL, tharg->engine, tharg->limits, tharg->options) == CL_VIRUS) {
+	    if(scan && cl_scanfile(acc->filename, &virname, NULL, tharg->engine, tharg->options) == CL_VIRUS) {
 		logg("Clamuko: %s: %s FOUND\n", acc->filename, virname);
-		virusaction(acc->filename, virname, tharg->copt);
+		virusaction(acc->filename, virname, tharg->opts);
 		acc->deny = 1;
 	    } else
 		acc->deny = 0;
