@@ -93,6 +93,8 @@ int scan_callback(struct stat *sb, char *filename, const char *msg, enum cli_ftw
     if (send(scandata->conn->sd, &ret, 0, 0) == -1 && errno != EINTR) {
 	logg("$Client disconnected while command was active!\n");
 	thrmgr_group_terminate(scandata->conn->group);
+	if (reason == visit_file)
+	    free(filename);
 	return CL_BREAK;
     }
 
@@ -143,20 +145,6 @@ int scan_callback(struct stat *sb, char *filename, const char *msg, enum cli_ftw
 	return CL_SUCCESS;
     }
 #endif
-    if((opt = optget(scandata->opts, "ExcludePath"))->enabled) {
-      while (opt) {
-	/* TODO: perhaps multiscan should skip this check? 
-	 * This should work unless the user is doing something stupid like
-	 * MULTISCAN / */
-	if(match_regex(filename, opt->strarg) == 1) {
-	    if (type != TYPE_MULTISCAN)
-		conn_reply_single(scandata->conn, filename, "Excluded");
-	    free(filename);
-	    return CL_SUCCESS;
-	}
-	opt = (const struct optstruct *) opt->nextarg;
-      }
-    }
 
     if(sb && sb->st_size == 0) { /* empty file */
 	if (msg == scandata->toplevel_path)
@@ -201,8 +189,10 @@ int scan_callback(struct stat *sb, char *filename, const char *msg, enum cli_ftw
     }
 
     if (access(filename, R_OK)) {
-	if (conn_reply(scandata->conn, filename, "Access denied.", "ERROR") == -1)
+	if (conn_reply(scandata->conn, filename, "Access denied.", "ERROR") == -1) {
+	    free(filename);
 	    return CL_ETIMEOUT;
+	}
 	logg("*Access denied: %s\n", filename);
 	scandata->errors++;
 	free(filename);
@@ -215,20 +205,25 @@ int scan_callback(struct stat *sb, char *filename, const char *msg, enum cli_ftw
     thrmgr_setactivetask(NULL, NULL);
 
     if (thrmgr_group_need_terminate(scandata->conn->group)) {
+	free(filename);
 	logg("*Client disconnected while scanjob was active\n");
 	return ret == CL_ETIMEOUT ? ret : CL_BREAK;
     }
 
     if (ret == CL_VIRUS) {
 	scandata->infected++;
-	if (conn_reply(scandata->conn, filename, virname, "FOUND") == -1)
+	if (conn_reply(scandata->conn, filename, virname, "FOUND") == -1) {
+	    free(filename);
 	    return CL_ETIMEOUT;
+	}
 	logg("~%s: %s FOUND\n", filename, virname);
 	virusaction(filename, virname, scandata->opts);
     } else if (ret != CL_CLEAN) {
 	scandata->errors++;
-	if (conn_reply(scandata->conn, filename, cl_strerror(ret), "ERROR") == -1)
+	if (conn_reply(scandata->conn, filename, cl_strerror(ret), "ERROR") == -1) {
+	    free(filename);
 	    return CL_ETIMEOUT;
+	}
 	logg("~%s: %s ERROR\n", filename, cl_strerror(ret));
     } else if (logok) {
 	logg("~%s: OK\n", filename);
@@ -247,6 +242,24 @@ int scan_callback(struct stat *sb, char *filename, const char *msg, enum cli_ftw
     return CL_SUCCESS;
 }
 
+int scan_pathchk(const char *path, struct cli_ftw_cbdata *data)
+{
+	struct scan_cb_data *scandata = data->data;
+	const struct optstruct *opt;
+
+    if((opt = optget(scandata->opts, "ExcludePath"))->enabled) {
+	while(opt) {
+	    if(match_regex(path, opt->strarg) == 1) {
+		if(scandata->type != TYPE_MULTISCAN)
+		    conn_reply_single(scandata->conn, path, "Excluded");
+		    return 1;
+	    }
+	    opt = (const struct optstruct *) opt->nextarg;
+	}
+    }
+    return 0;
+}
+
 int scanfd(const int fd, const client_conn_t *conn, unsigned long int *scanned,
 	   const struct cl_engine *engine,
 	   unsigned int options, const struct optstruct *opts, int odesc, int stream)
