@@ -718,7 +718,7 @@ int recvloop_th(int *socketds, unsigned nsockets, struct cl_engine *engine, unsi
 #endif
 
     /* set up limits */
-    if((opt = optget(opts, "MaxScanSize"))->enabled) {
+    if((opt = optget(opts, "MaxScanSize"))->active) {
 	if((ret = cl_engine_set_num(engine, CL_ENGINE_MAX_SCANSIZE, opt->numarg))) {
 	    logg("!cl_engine_set_num(CL_ENGINE_MAX_SCANSIZE) failed: %s\n", cl_strerror(ret));
 	    cl_engine_free(engine);
@@ -731,7 +731,7 @@ int recvloop_th(int *socketds, unsigned nsockets, struct cl_engine *engine, unsi
     else
     	logg("^Limits: Global size limit protection disabled.\n");
 
-    if((opt = optget(opts, "MaxFileSize"))->enabled) {
+    if((opt = optget(opts, "MaxFileSize"))->active) {
 	if((ret = cl_engine_set_num(engine, CL_ENGINE_MAX_FILESIZE, opt->numarg))) {
 	    logg("!cl_engine_set_num(CL_ENGINE_MAX_FILESIZE) failed: %s\n", cl_strerror(ret));
 	    cl_engine_free(engine);
@@ -755,7 +755,7 @@ int recvloop_th(int *socketds, unsigned nsockets, struct cl_engine *engine, unsi
     }
 #endif
 
-    if((opt = optget(opts, "MaxRecursion"))->enabled) {
+    if((opt = optget(opts, "MaxRecursion"))->active) {
 	if((ret = cl_engine_set_num(engine, CL_ENGINE_MAX_RECURSION, opt->numarg))) {
 	    logg("!cl_engine_set_num(CL_ENGINE_MAX_RECURSION) failed: %s\n", cl_strerror(ret));
 	    cl_engine_free(engine);
@@ -768,7 +768,7 @@ int recvloop_th(int *socketds, unsigned nsockets, struct cl_engine *engine, unsi
     else
     	logg("^Limits: Recursion level limit protection disabled.\n");
 
-    if((opt = optget(opts, "MaxFiles"))->enabled) {
+    if((opt = optget(opts, "MaxFiles"))->active) {
 	if((ret = cl_engine_set_num(engine, CL_ENGINE_MAX_FILES, opt->numarg))) {
 	    logg("!cl_engine_set_num(CL_ENGINE_MAX_FILES) failed: %s\n", cl_strerror(ret));
 	    cl_engine_free(engine);
@@ -927,7 +927,7 @@ int recvloop_th(int *socketds, unsigned nsockets, struct cl_engine *engine, unsi
     mainpid = getpid();
     if((opt = optget(opts, "PidFile"))->enabled) {
 	    FILE *fd;
-	old_umask = umask(0006);
+	old_umask = umask(0002);
 	if((fd = fopen(opt->strarg, "w")) == NULL) {
 	    logg("!Can't save PID in file %s\n", opt->strarg);
 	} else {
@@ -1258,6 +1258,13 @@ int recvloop_th(int *socketds, unsigned nsockets, struct cl_engine *engine, unsi
 	pthread_mutex_lock(&reload_mutex);
 	if(reload) {
 	    pthread_mutex_unlock(&reload_mutex);
+#ifdef CLAMUKO
+	    if(optget(opts, "ClamukoScanOnAccess")->enabled && tharg) {
+		logg("Stopping and restarting Clamuko.\n");
+		pthread_kill(clamuko_pid, SIGUSR1);
+		pthread_join(clamuko_pid, NULL);
+	    }
+#endif
 	    engine = reload_db(engine, dboptions, opts, FALSE, &ret);
 	    if(ret) {
 		logg("Terminating because of a fatal error.\n");
@@ -1272,9 +1279,6 @@ int recvloop_th(int *socketds, unsigned nsockets, struct cl_engine *engine, unsi
 	    pthread_mutex_unlock(&reload_mutex);
 #ifdef CLAMUKO
 	    if(optget(opts, "ClamukoScanOnAccess")->enabled && tharg) {
-		logg("Stopping and restarting Clamuko.\n");
-		pthread_kill(clamuko_pid, SIGUSR1);
-		pthread_join(clamuko_pid, NULL);
 		tharg->engine = engine;
 		pthread_create(&clamuko_pid, &clamuko_attr, clamukoth, tharg);
 	    }
