@@ -1,5 +1,7 @@
 /*
- *  Copyright (C) 2002 - 2006 Tomasz Kojm <tkojm@clamav.net>
+ *  Copyright (C) 2007-2009 Sourcefire, Inc.
+ *
+ *  Authors: Tomasz Kojm, aCaB
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -28,19 +30,16 @@
 #include <time.h>
 #include <signal.h>
 
-#include "options.h"
-#include "others.h"
-#include "defaults.h"
-#include "client.h"
-#include "output.h"
-#include "misc.h"
+#include "shared/output.h"
+#include "shared/misc.h"
+#include "shared/optparser.h"
+#include "shared/actions.h"
 
-#include "clamscan/clamscan_opt.h"
+#include "client.h"
 
 void help(void);
 
 extern int printinfected;
-extern int notremoved, notmoved;
 
 static void print_server_version(const struct optstruct *opt)
 {
@@ -50,90 +49,88 @@ static void print_server_version(const struct optstruct *opt)
     }
 }
 
-static int reload_server_database(const struct optstruct *opt)
-{
-    if(reload_clamd_database(opt)) {
-	logg("!Clamd did not reload the database\n");
-	return 2;
-    }
-    return 0;
-}
-
 int main(int argc, char **argv)
 {
 	int ds, dms, ret, infected;
 	struct timeval t1, t2;
 	struct timezone tz;
 	time_t starttime;
-	struct optstruct *opt;
-	const char *clamdscan_accepted[] = { "help", "version", "verbose", "quiet",
-				  "stdout", "log", "move", "copy", "remove",
-				  "config-file", "no-summary",  "fdpass",
-				  "disable-summary", "multiscan", "reload",
-				  "infected",
-				  NULL };
+        struct optstruct *opts;
+        const struct optstruct *opt;
+	struct sigaction sigact;
 
 
-    opt = opt_parse(argc, argv, clamscan_shortopt, clamscan_longopt, clamdscan_accepted, NULL);
-    if(!opt) {
-	mprintf("!Can't parse the command line\n");
+    if((opts = optparse(NULL, argc, argv, 1, OPT_CLAMDSCAN, OPT_CLAMSCAN, NULL)) == NULL) {
+	mprintf("!Can't parse command line options\n");
 	return 2;
     }
 
-    if(opt_check(opt, "verbose")) {
+    if(optget(opts, "verbose")->enabled) {
 	mprintf_verbose = 1;
 	logg_verbose = 1;
     }
 
-    if(opt_check(opt, "quiet"))
+    if(optget(opts, "quiet")->enabled)
 	mprintf_quiet = 1;
 
-    if(opt_check(opt, "stdout"))
+    if(optget(opts, "stdout")->enabled)
 	mprintf_stdout = 1;
 
-    if(opt_check(opt, "version")) {
-	print_server_version(opt);
-	opt_free(opt);
+    if(optget(opts, "version")->enabled) {
+	print_server_version(opts);
+	optfree(opts);
 	exit(0);
     }
 
-    if(opt_check(opt, "help")) {
-	opt_free(opt);
+    if(optget(opts, "help")->enabled) {
+	optfree(opts);
     	help();
     }
 
-    if(opt_check(opt, "infected"))
+    if(optget(opts, "infected")->enabled)
 	printinfected = 1;
 
     /* initialize logger */
 
-    if(opt_check(opt, "log")) {
-	logg_file = opt_arg(opt, "log");
+    if((opt = optget(opts, "log"))->enabled) {
+	logg_file = opt->strarg;
 	if(logg("--------------------------------------\n")) {
 	    mprintf("!Problem with internal logger.\n");
-	    opt_free(opt);
+	    optfree(opts);
 	    exit(2);
 	}
     } else 
 	logg_file = NULL;
 
 
-    if(opt_check(opt, "reload")) {
-	ret = reload_server_database(opt);
-	opt_free(opt);
+    if(optget(opts, "reload")->enabled) {
+	ret = reload_clamd_database(opts);
+	optfree(opts);
 	logg_close();
 	exit(ret);
     }
 
+    if(actsetup(opts)) {
+	optfree(opts);
+	logg_close();
+	exit(2);
+    }
+
+    memset(&sigact, 0, sizeof(struct sigaction));
+    sigact.sa_handler = SIG_IGN;
+    sigemptyset(&sigact.sa_mask);
+    sigaddset(&sigact.sa_mask, SIGPIPE);
+    sigaction(SIGPIPE, &sigact, NULL);
+
     time(&starttime);
     /* ctime() does \n, but I need it once more */
 
     gettimeofday(&t1, &tz);
 
-    ret = client(opt, &infected);
+    ret = client(opts, &infected);
 
     /* TODO: Implement STATUS in clamd */
-    if((infected || ret != 2) && !opt_check(opt, "disable-summary") && !opt_check(opt, "no-summary")) {
+    if((infected || ret != 2) && !optget(opts, "no-summary")->enabled) {
 	gettimeofday(&t2, &tz);
 	ds = t2.tv_sec - t1.tv_sec;
 	dms = t2.tv_usec - t1.tv_usec;
@@ -151,7 +148,7 @@ int main(int argc, char **argv)
     }
 
     logg_close();
-    opt_free(opt);
+    optfree(opts);
     exit(ret);
 }
 
@@ -162,7 +159,8 @@ void help(void)
 
     mprintf("\n");
     mprintf("                       ClamAV Daemon Client %s\n", get_version());
-    mprintf("     (C) 2002 - 2007 ClamAV Team - http://www.clamav.net/team\n\n");
+    printf("           By The ClamAV Team: http://www.clamav.net/team\n");
+    printf("           (C) 2007-2009 Sourcefire, Inc.\n\n");
 
     mprintf("    --help              -h             Show help\n");
     mprintf("    --version           -V             Print version number and exit\n");
@@ -171,6 +169,7 @@ void help(void)
     mprintf("    --stdout                           Write to stdout instead of stderr\n");
     mprintf("                                       (this help is always written to stdout)\n");
     mprintf("    --log=FILE          -l FILE        Save scan report in FILE\n");
+    mprintf("    --file-list=FILE    -f FILE        Scan files from FILE\n");
     mprintf("    --remove                           Remove infected files. Be careful!\n");
     mprintf("    --move=DIRECTORY                   Move infected files into DIRECTORY\n");
     mprintf("    --copy=DIRECTORY                   Copy infected files into DIRECTORY\n");
@@ -179,7 +178,8 @@ void help(void)
     mprintf("    --infected            -i           Only print infected files\n");
     mprintf("    --no-summary                       Disable summary at end of scanning\n");
     mprintf("    --reload                           Request clamd to reload virus database\n");
-    mprintf("    --fdpass                           pass filedescriptor to clamd (useful if clamd is running as a different user)\n");
+    mprintf("    --fdpass                           Pass filedescriptor to clamd (useful if clamd is running as a different user)\n");
+    mprintf("    --stream                           Force streaming files to clamd (for debugging and unit testing)\n");
     mprintf("\n");
 
     exit(0);
