@@ -28,6 +28,10 @@
 #include "clamav-config.h"
 #endif
 
+#ifdef HAVE_STDINT_H
+#include <stdint.h>
+#endif
+
 #include <unistd.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -1147,7 +1151,7 @@ static void setup_connections(int argc, char *argv[])
 		if ((opt_addr = optget(clamd_opts, "TCPAddr"))->enabled) {
 		    host = opt_addr->strarg;
 		}
-		snprintf(buf, sizeof(buf), "%s:%u", host, opt->numarg);
+		snprintf(buf, sizeof(buf), "%s:%llu", host, opt->numarg);
 		conn = strdup(buf);
 	    } else {
 		fprintf(stderr, "Can't find how to connect to clamd\n");
@@ -1187,6 +1191,7 @@ static void setup_connections(int argc, char *argv[])
 		}
 	}
 	optfree(opts);
+	free(conn);
 #ifndef _WIN32
 	signal(SIGPIPE, SIG_IGN);
 	signal(SIGINT, sigint);
