@@ -1066,7 +1066,7 @@ static int getfile(const char *srcfile, const char *destfile, const char *hostna
 #else
 	if((i >= sizeof(buffer) - 1) || recv(sd, buffer + i, 1, 0) == -1) {
 #endif
-	    logg("%cgetfile: Error while reading database from %s (IP: %s)\n", logerr ? '!' : '^', hostname, ipaddr);
+	    logg("%cgetfile: Error while reading database from %s (IP: %s): %s\n", logerr ? '!' : '^', hostname, ipaddr, strerror(errno));
 	    mirman_update(mdat->currip, mdat->af, mdat, 1);
 	    closesocket(sd);
 	    return 52;
@@ -1156,6 +1156,12 @@ static int getfile(const char *srcfile, const char *destfile, const char *hostna
     closesocket(sd);
     close(fd);
 
+    if(bread == -1) {
+	logg("%cgetfile: Download interrupted: %s (IP: %s)\n", logerr ? '!' : '^', strerror(errno), ipaddr);
+	mirman_update(mdat->currip, mdat->af, mdat, 1);
+	return 52;
+    }
+
     if(totalsize > 0)
         logg("Downloading %s [%i%%]\n", srcfile, percentage);
     else
@@ -1409,6 +1415,7 @@ static int buildcld(const char *tmpdir, const char *dbname, const char *newfile,
 	else
 	    close(fd);
 	unlink(newfile);
+	closedir(dir);
 	return -1;
     }
 
@@ -1677,18 +1684,12 @@ static int updatedb(const char *dbname, const char *hostname, char *ip, int *sig
 	return 55; /* FIXME */
     }
 
-    if(!nodb && !access(localname, R_OK) && unlink(localname)) {
-	logg("!Can't unlink %s. Please fix it and try again.\n", localname);
-	unlink(newfile);
-	free(newfile);
-	return 53;
-    }
-
 #ifdef C_WINDOWS
     if(!access(newdb, R_OK) && unlink(newdb)) {
 	logg("!Can't unlink %s. Please fix the problem manually and try again.\n", newdb);
 	unlink(newfile);
 	free(newfile);
+	cl_cvdfree(current);
 	return 53;
     }
 #endif
@@ -1697,10 +1698,15 @@ static int updatedb(const char *dbname, const char *hostname, char *ip, int *sig
 	logg("!Can't rename %s to %s: %s\n", newfile, newdb, strerror(errno));
 	unlink(newfile);
 	free(newfile);
+	cl_cvdfree(current);
 	return 57;
     }
     free(newfile);
 
+    if(!nodb && !access(localname, R_OK) && strcmp(newdb, localname))
+	if(unlink(localname))
+	    logg("^Can't unlink the old database file %s. Please remove it manually.\n", localname);
+
     logg("%s updated (version: %d, sigs: %d, f-level: %d, builder: %s)\n", newdb, current->version, current->sigs, current->fl, current->builder);
 
     if(flevel < current->fl) {
@@ -1918,7 +1924,7 @@ int downloadmanager(const struct optstruct *opts, const char *hostname, const ch
 	    }
 
 	    if(newver)
-		execute("OnOutdatedExecute", cmd, opt);
+		execute("OnOutdatedExecute", cmd, opts);
 
 	    free(cmd);
 	}
