@@ -88,6 +88,13 @@ static const struct ftmap_s {
     { "CL_TYPE_ARJSFX",		CL_TYPE_ARJSFX		},
     { "CL_TYPE_NULSFT",		CL_TYPE_NULSFT		},
     { "CL_TYPE_AUTOIT",		CL_TYPE_AUTOIT		},
+
+    /* for compatibility with daily.cvd < 9954 */
+    { "CL_TYPE_CPIO_NEWC",	CL_TYPE_CPIO_NEWC	},
+    { "CL_TYPE_CPIO_CRC",	CL_TYPE_CPIO_CRC	},
+    { "CL_TYPE_CPIO_ODC",	CL_TYPE_CPIO_ODC	},
+    { "CL_TYPE_CPIO_OLD",	CL_TYPE_CPIO_OLD	},
+
     { NULL,			CL_TYPE_IGNORED		}
 };
 
@@ -209,7 +216,6 @@ cli_file_t cli_filetype2(int desc, const struct cl_engine *engine)
 			     * However when detecting whether a file is HTML or not, we need exact conversion.
 			     * (just eliminating zeros and matching would introduce false positives */
 			    if(encoding_normalize_toascii(&in_area, encoding, &out_area) >= 0 && out_area.length > 0) {
-				    out_area.buffer[out_area.length] = '\0';
 				    if(cli_ac_initdata(&mdata, root->ac_partsigs, root->ac_lsigs, CLI_DEFAULT_AC_TRACKLEN))
 					    return ret;
 
