@@ -89,7 +89,7 @@ enum fsm_state {
 };
 
 struct scope {
-	struct hashtable id_map;
+	struct cli_hashtable id_map;
 	struct scope *parent;/* hierarchy */
 	struct scope *nxt;/* all scopes kept in a list so we can easily free all of them */
 	enum fsm_state fsm_state;
@@ -122,7 +122,7 @@ static struct scope* scope_new(struct parser_state *state)
 	struct scope *s = cli_calloc(1, sizeof(*s));
 	if(!s)
 		return NULL;
-	if(hashtab_init(&s->id_map, 10) < 0) {
+	if(cli_hashtab_init(&s->id_map, 10) < 0) {
 		free(s);
 		return NULL;
 	}
@@ -138,7 +138,7 @@ static struct scope* scope_done(struct scope *s)
 {
 	struct scope* parent = s->parent;
 	/* TODO: have a hashtab_destroy */
-	hashtab_clear(&s->id_map);
+	cli_hashtab_clear(&s->id_map);
 	free(s->id_map.htable);
 	free(s);
 	return parent;
@@ -250,15 +250,15 @@ static struct scope* scope_done(struct scope *s)
 
 static const char* scope_declare(struct scope *s, const char *token, const size_t len, struct parser_state *state)
 {
-	const struct element *el = hashtab_insert(&s->id_map, token, len, state->var_uniq++);
-	/* hashtab_insert either finds an already existing entry, or allocates a
+	const struct cli_element *el = cli_hashtab_insert(&s->id_map, token, len, state->var_uniq++);
+	/* cli_hashtab_insert either finds an already existing entry, or allocates a
 	 * new one, we return the allocated string */
 	return el ? el->key : NULL;
 }
 
 static const char* scope_use(struct scope *s, const char *token, const size_t len)
 {
-	const struct element *el = hashtab_find(&s->id_map, token, len);
+	const struct cli_element *el = cli_hashtab_find(&s->id_map, token, len);
 	if(el) {
 		/* identifier already found in current scope,
 		 * return here to avoid overwriting uniq id */
@@ -268,14 +268,14 @@ static const char* scope_use(struct scope *s, const char *token, const size_t le
 	 * Later if we find a declaration it will automatically assign a uniq ID
 	 * to it. If not, we'll know that we have to push ID == -1 tokens to an
 	 * outer scope.*/
-	el = hashtab_insert(&s->id_map, token, len, -1);
+	el = cli_hashtab_insert(&s->id_map, token, len, -1);
 	return el ? el->key : NULL;
 }
 
 static long scope_lookup(struct scope *s, const char *token, const size_t len)
 {
 	while(s) {
-		const struct element *el = hashtab_find(&s->id_map, token, len);
+		const struct cli_element *el = cli_hashtab_find(&s->id_map, token, len);
 		if(el && el->data != -1) {
 			return el->data;
 		}
@@ -288,10 +288,13 @@ static long scope_lookup(struct scope *s, const char *token, const size_t len)
 static int tokens_ensure_capacity(struct tokens *tokens, size_t cap)
 {
 	if(tokens->capacity < cap) {
+	        yystype *data;
 		cap += 1024;
-		tokens->data = cli_realloc(tokens->data, cap * sizeof(*tokens->data));
-		if(!tokens->data)
+		/* Keep old data if OOM */
+		data = cli_realloc(tokens->data, cap * sizeof(*tokens->data));
+		if(!data)
 			return CL_EMEM;
+		tokens->data = data;
 		tokens->capacity = cap;
 	}
 	return CL_SUCCESS;
@@ -299,7 +302,7 @@ static int tokens_ensure_capacity(struct tokens *tokens, size_t cap)
 
 static int add_token(struct parser_state *state, const yystype *token)
 {
-	if(tokens_ensure_capacity(&state->tokens, state->tokens.cnt + 1) < 0)
+	if(tokens_ensure_capacity(&state->tokens, state->tokens.cnt + 1))
 		return -1;
 	state->tokens.data[state->tokens.cnt++] = *token;
 	return 0;
@@ -502,15 +505,14 @@ static const char *de_packer_2[] = {"p","a","c","k","e","d"};
 
 static inline char *textbuffer_done(yyscan_t scanner)
 {
-	/* free unusued memory */
-	char *str = cli_realloc(scanner->buf.data, scanner->buf.pos);
-	if(!str) {
-		str = scanner->buf.data;
-	}
-	scanner->yytext = str;
-	scanner->yylen = scanner->buf.pos - 1;
-	memset(&scanner->buf, 0, sizeof(scanner->buf));
-	return str;
+       char *str = cli_realloc(scanner->buf.data, scanner->buf.pos);
+       if(!str) {
+               str = scanner->buf.data;
+       }
+       scanner->yytext = str;
+       scanner->yylen = scanner->buf.pos - 1;
+       memset(&scanner->buf, 0, sizeof(scanner->buf));
+       return str;
 }
 
 #define MODULE "JS-Norm: "
@@ -533,7 +535,7 @@ static int replace_token_range(struct tokens *dst, size_t start, size_t end, con
 	for(i=start;i<end;i++) {
 		free_token(&dst->data[i]);
 	}
-	if(tokens_ensure_capacity(dst, dst->cnt - (end-start) + len) < 0)
+	if(tokens_ensure_capacity(dst, dst->cnt - (end-start) + len))
 		return CL_EMEM;
 	memmove(&dst->data[start+len], &dst->data[end], (dst->cnt - end) * sizeof(dst->data[0]));
 	if(with && len > 0) {
@@ -547,7 +549,7 @@ static int append_tokens(struct tokens *dst, const struct tokens *src)
 {
 	if(!dst || !src)
 		return CL_ENULLARG;
-	if(tokens_ensure_capacity(dst, dst->cnt + src->cnt) == -1)
+	if(tokens_ensure_capacity(dst, dst->cnt + src->cnt))
 		return CL_EMEM;
 	cli_dbgmsg(MODULE "Appending %lu tokens\n", src->cnt);
 	memcpy(&dst->data[dst->cnt], src->data, src->cnt * sizeof(dst->data[0]));
@@ -1127,6 +1129,8 @@ void cli_js_process_buffer(struct parser_state *state, const char *buf, size_t n
 						free_token(&state->tokens.data[--state->tokens.cnt]);
 
 						str = cli_realloc(str, str_len + leng + 1);
+						if (!str)
+						    break;
 						strncpy(str+str_len, text, leng);
 						str[str_len + leng] = '\0';
 						TOKEN_SET(prev_string, string, str);
@@ -1254,7 +1258,9 @@ static const enum char_class id_ctype[256] = {
 static void textbuf_clean(struct text_buffer *buf)
 {
 	if(buf->capacity > BUF_KEEP_SIZE) {
-		buf->data = cli_realloc(buf->data, BUF_KEEP_SIZE);
+	        char *data= cli_realloc(buf->data, BUF_KEEP_SIZE);
+		if (data)
+		    buf->data = data;
 		buf->capacity = BUF_KEEP_SIZE;
 	}
 	buf->pos = 0;
@@ -1281,10 +1287,16 @@ static inline int parseString(YYSTYPE *lvalp, yyscan_t scanner, const char q,
 		len = scanner->insize - scanner->pos;
 	cli_textbuffer_append_normalize(&scanner->buf, start, len);
 	if(end) {
+	        char *str;
 		/* skip over end quote */
 		scanner->pos += len + 1;
 		textbuffer_putc(&scanner->buf, '\0');
-		TOKEN_SET(lvalp, string, textbuffer_done(scanner));
+		str = textbuffer_done(scanner);
+		if (str) {
+		    TOKEN_SET(lvalp, string, str);
+		} else {
+		    TOKEN_SET(lvalp, cstring, "");
+		}
 		scanner->state = Initial;
 		assert(lvalp->val.string);
 		return TOK_StringLiteral;
@@ -1334,6 +1346,8 @@ static inline int parseNumber(YYSTYPE *lvalp, yyscan_t scanner)
 		scanner->pos--;
 		textbuffer_putc(&scanner->buf, '\0');
 		scanner->state = Initial;
+		if (!scanner->buf.data)
+			return 0;
 		if(is_float) {
 			TOKEN_SET(lvalp, dval, atof(scanner->buf.data));
 			return TOK_NumericFloat;
@@ -1438,14 +1452,13 @@ static int yy_scan_bytes(const char *p, size_t len, yyscan_t scanner)
 
 static const char *yyget_text(yyscan_t scanner)
 {
-	assert(scanner->buf.data || scanner->yytext);
-	return scanner->yytext ? scanner->yytext : scanner->buf.data;
+    return scanner->yytext ? scanner->yytext :  scanner->buf.data;
 }
 
 static int yyget_leng(yyscan_t scanner)
 {
 	/* we have a \0 too */
-	return scanner->yylen ? scanner->yylen : (scanner->buf.pos > 0 ? scanner->buf.pos - 1 : 0);
+	return scanner->yylen ? scanner->yylen: (scanner->buf.pos > 0 ? scanner->buf.pos - 1 : 0);
 }
 
 static int yylex(YYSTYPE *lvalp, yyscan_t  scanner)
