@@ -26,6 +26,7 @@
 #include <string.h>
 #include <stdlib.h>
 #include <ctype.h>
+#include <sys/stat.h>
 
 #include <assert.h>
 #ifdef	HAVE_UNISTD_H
@@ -1010,7 +1011,7 @@ int cli_ac_scanbuff(const unsigned char *buffer, uint32_t length, const char **v
 				} else { /* !pt->type */
 				    if(pt->lsigid[0]) {
 					mdata->lsigcnt[pt->lsigid[1]][pt->lsigid[2]]++;
-					pt = pt->next;
+					pt = pt->next_same;
 					continue;
 				    }
 
@@ -1026,7 +1027,7 @@ int cli_ac_scanbuff(const unsigned char *buffer, uint32_t length, const char **v
 					newres->next = *res;
 					*res = newres;
 
-					pt = pt->next;
+					pt = pt->next_same;
 					continue;
 				    } else {
 					if(virname)
@@ -1065,7 +1066,7 @@ int cli_ac_scanbuff(const unsigned char *buffer, uint32_t length, const char **v
 			    } else {
 				if(pt->lsigid[0]) {
 				    mdata->lsigcnt[pt->lsigid[1]][pt->lsigid[2]]++;
-				    pt = pt->next;
+				    pt = pt->next_same;
 				    continue;
 				}
 
@@ -1081,7 +1082,7 @@ int cli_ac_scanbuff(const unsigned char *buffer, uint32_t length, const char **v
 				    newres->next = *res;
 				    *res = newres;
 
-				    pt = pt->next;
+				    pt = pt->next_same;
 				    continue;
 				} else {
 				    if(virname)
