@@ -45,7 +45,7 @@
  * in re-enabling affected modules.
  */
 
-#define CL_FLEVEL 43
+#define CL_FLEVEL 44
 #define CL_FLEVEL_DCONF	CL_FLEVEL
 
 extern uint8_t cli_debug_flag;
@@ -306,7 +306,7 @@ static inline void cli_writeint32(char *offset, uint32_t value)
 #ifdef HAVE_SAR
 #define CLI_SRS(n,s) ((n)>>(s))
 #else
-#define CLI_SRS(n,s) (((n)>>(s)) ^ (1<<(sizeof(n)*8-1-s)) - (1<<(sizeof(n)*8-1-s)))
+#define CLI_SRS(n,s) ((((n)>>(s)) ^ (1<<(sizeof(n)*8-1-s))) - (1<<(sizeof(n)*8-1-s)))
 #endif
 #define CLI_SAR(n,s) n = CLI_SRS(n,s)
 
@@ -359,6 +359,17 @@ void cli_dbgmsg_internal(const char *str, ...) __attribute__((format(printf, 1,
 void cli_dbgmsg_internal(const char *str, ...);
 #endif
 
+#if HAVE_SYSCONF_SC_PAGESIZE
+static inline int cli_getpagesize(void) { return sysconf(_SC_PAGESIZE); }
+#define HAVE_CLI_GETPAGESIZE 1
+#else
+#if HAVE_GETPAGESIZE
+static inline int cli_getpagesize(void) { return getpagesize(); }
+#define HAVE_CLI_GETPAGESIZE 1
+#endif
+#define HAVE_CLI_GETPAGESIZE 0
+#endif
+
 void *cli_malloc(size_t nmemb);
 void *cli_calloc(size_t nmemb, size_t size);
 void *cli_realloc(void *ptr, size_t size);
@@ -422,6 +433,12 @@ struct cli_ftw_cbdata {
  */
 typedef int (*cli_ftw_cb)(struct stat *stat_buf, char *filename, const char *path, enum cli_ftw_reason reason, struct cli_ftw_cbdata *data);
 
+/*
+ * returns 1 if the path should be skipped and 0 otherwise
+ * uses callback data
+ */
+typedef int (*cli_ftw_pathchk)(const char *path, struct cli_ftw_cbdata *data);
+
 /*
  * returns 
  *  CL_SUCCESS if it traversed all files and subdirs
@@ -436,7 +453,7 @@ typedef int (*cli_ftw_cb)(struct stat *stat_buf, char *filename, const char *pat
  * which one it is.
  * If it is a file, it simply calls the callback once, otherwise recurses.
  */
-int cli_ftw(char *base, int flags, int maxdepth, cli_ftw_cb callback, struct cli_ftw_cbdata *data);
+int cli_ftw(char *base, int flags, int maxdepth, cli_ftw_cb callback, struct cli_ftw_cbdata *data, cli_ftw_pathchk pathchk);
 
 const char *cli_strerror(int errnum, char* buf, size_t len);
 #endif
