@@ -24,11 +24,6 @@
 #include "clamav-config.h"
 #endif
 
-
-#ifndef CL_DEBUG
-#define NDEBUG
-#endif
-
 #ifdef CL_THREAD_SAFE
 #ifndef _REENTRANT
 #define _REENTRANT
@@ -45,7 +40,7 @@
 #include "phish_domaincheck_db.h"
 #include "regex_list.h"
 
-int domainlist_match(const struct cl_engine* engine,char* real_url,const char* display_url,const struct pre_fixup_info* pre_fixup,int hostOnly,unsigned short* flags)
+int domainlist_match(const struct cl_engine* engine,char* real_url,const char* display_url,const struct pre_fixup_info* pre_fixup,int hostOnly)
 {
 	const char* info;
 	int rc = engine->domainlist_matcher ? regex_list_match(engine->domainlist_matcher,real_url,display_url,hostOnly ? pre_fixup : NULL,hostOnly,&info,0) : 0;
@@ -58,6 +53,9 @@ int init_domainlist(struct cl_engine* engine)
 		engine->domainlist_matcher = (struct regex_matcher *) cli_malloc(sizeof(struct regex_matcher));
 		if(!engine->domainlist_matcher)
 			return CL_EMEM;
+#ifdef USE_MPOOL
+		((struct regex_matcher*)engine->domainlist_matcher)->mempool = engine->mempool;
+#endif
 		return init_regex_list(engine->domainlist_matcher);
 	}
 	else
