@@ -24,10 +24,6 @@
 #include "clamav-config.h"
 #endif
 
-#ifndef CL_DEBUG
-#define NDEBUG
-#endif
-
 #ifdef CL_THREAD_SAFE
 #ifndef _REENTRANT
 #define _REENTRANT
@@ -57,6 +53,11 @@
 #include "readdb.h"
 #include "jsparse/textbuf.h"
 #include "regex_suffix.h"
+#include "default.h"
+#include "hashtab.h"
+
+#include "mpool.h"
+
 /* Prototypes */
 static regex_t *new_preg(struct regex_matcher *matcher);
 static size_t reverse_string(char *pattern);
@@ -272,7 +273,7 @@ int regex_list_match(struct regex_matcher* matcher,char* real_url,const char* di
 	{
 		char *buffer = cli_malloc(buffer_len+1);
 		char *bufrev;
-		int rc = 0;
+		int rc = 0, root;
 		struct cli_ac_data mdata;
 		struct cli_ac_result *res = NULL;
 
@@ -288,7 +289,7 @@ int regex_list_match(struct regex_matcher* matcher,char* real_url,const char* di
 		buffer[buffer_len]=0;
 		cli_dbgmsg("Looking up in regex_list: %s\n", buffer);
 
-		if((rc = cli_ac_initdata(&mdata, 0, 0, AC_DEFAULT_TRACKLEN)))
+		if((rc = cli_ac_initdata(&mdata, 0, 0, CLI_DEFAULT_AC_TRACKLEN)))
 			return rc;
 
 		bufrev = cli_strdup(buffer);
@@ -309,9 +310,15 @@ int regex_list_match(struct regex_matcher* matcher,char* real_url,const char* di
 		cli_ac_freedata(&mdata);
 
 		rc = 0;
-		while(res) {
+		root = matcher->root_regex_idx;
+		while(res || root) {
 			struct cli_ac_result *q;
-			regex = res->customdata;
+			if (!res) {
+			    regex = matcher->suffix_regexes[root].head;
+			    root = 0;
+			} else {
+			    regex = res->customdata;
+			}
 			while(!rc && regex) {
 				/* loop over multiple regexes corresponding to
 				 * this suffix */
@@ -324,9 +331,11 @@ int regex_list_match(struct regex_matcher* matcher,char* real_url,const char* di
 				if(rc) *info = regex->pattern;
 				regex = regex->nxt;
 			}
-			q = res;
-			res = res->next;
-			free(q);
+			if (res) {
+			    q = res;
+			    res = res->next;
+			    free(q);
+			}
 		}
 		free(buffer);
 		if(!rc)
@@ -342,6 +351,9 @@ int regex_list_match(struct regex_matcher* matcher,char* real_url,const char* di
 /* Initializes @matcher, allocating necesarry substructures */
 int init_regex_list(struct regex_matcher* matcher)
 {
+#ifdef USE_MPOOL
+	mpool_t *mp = matcher->mempool;
+#endif
 	int rc;
 
 	assert(matcher);
@@ -350,16 +362,26 @@ int init_regex_list(struct regex_matcher* matcher)
 	matcher->list_inited=1;
 	matcher->list_built=0;
 	matcher->list_loaded=0;
-
-	hashtab_init(&matcher->suffix_hash, 10);
+	cli_hashtab_init(&matcher->suffix_hash, 512);
+#ifdef USE_MPOOL
+	matcher->mempool = mp;
+	matcher->suffixes.mempool = mp;
+	assert(mp && "mempool must be initialized");
+#endif
 	if((rc = cli_ac_init(&matcher->suffixes, 2, 32))) {
 		return rc;
 	}
-	if((rc = cli_bm_init(&matcher->md5_hashes))) {
+#ifdef USE_MPOOL
+	matcher->sha256_hashes.mempool = mp;
+	matcher->hostkey_prefix.mempool = mp;
+#endif
+	if((rc = cli_bm_init(&matcher->sha256_hashes))) {
+		return rc;
+	}
+	if((rc = cli_bm_init(&matcher->hostkey_prefix))) {
 		return rc;
 	}
 	SO_init(&matcher->filter);
-	SO_init(&matcher->md5_filter);
 	return CL_SUCCESS;
 }
 
@@ -406,24 +428,50 @@ static int functionality_level_check(char* line)
 	}
 }
 
-static int add_hash(struct regex_matcher *matcher, char* pattern, const char fl)
+static int add_hash(struct regex_matcher *matcher, char* pattern, const char fl, int is_prefix)
 {
 	int rc;
-	struct cli_bm_patt *pat = cli_calloc(1, sizeof(*pat));
+	struct cli_bm_patt *pat = mpool_calloc(matcher->mempool, 1, sizeof(*pat));
+	struct cli_matcher *bm;
+	const char *vname = NULL;
 	if(!pat)
 		return CL_EMEM;
-	pat->pattern = (unsigned char*)cli_hex2str(pattern);
+	pat->pattern = (unsigned char*)cli_mpool_hex2str(matcher->mempool, pattern);
 	if(!pat->pattern)
 		return CL_EMALFDB;
-	pat->length = 16;
-	pat->virname = cli_malloc(1);
+	pat->length = 32;
+	if (is_prefix) {
+	    pat->length=4;
+	    bm = &matcher->hostkey_prefix;
+	} else {
+	    bm = &matcher->sha256_hashes;
+	}
+
+	if (!matcher->sha256_pfx_set.keys) {
+	    if((rc = cli_hashset_init(&matcher->sha256_pfx_set, 1048576, 90))) {
+		return rc;
+	    }
+	}
+
+	if (fl != 'W' && pat->length == 32 &&
+	    cli_hashset_contains(&matcher->sha256_pfx_set, cli_readint32(pat->pattern)) &&
+	    cli_bm_scanbuff(pat->pattern, 32, &vname, NULL, &matcher->sha256_hashes,0,0,-1) == CL_VIRUS) {
+	    if (*vname == 'W') {
+		/* hash is whitelisted in local.gdb */
+		cli_dbgmsg("Skipping hash %s\n", pattern);
+		mpool_free(matcher->mempool, pat->pattern);
+		mpool_free(matcher->mempool, pat);
+		return CL_SUCCESS;
+	    }
+	}
+	pat->virname = mpool_malloc(matcher->mempool, 1);
 	if(!pat->virname) {
 		free(pat);
 		return CL_EMEM;
 	}
 	*pat->virname = fl;
-	SO_preprocess_add(&matcher->md5_filter, pat->pattern, pat->length);
-	if((rc = cli_bm_addpatt(&matcher->md5_hashes, pat))) {
+	cli_hashset_addkey(&matcher->sha256_pfx_set, cli_readint32(pat->pattern));
+	if((rc = cli_bm_addpatt(bm, pat))) {
 		cli_errmsg("add_hash: failed to add BM pattern\n");
 		free(pat->pattern);
 		free(pat->virname);
@@ -435,9 +483,9 @@ static int add_hash(struct regex_matcher *matcher, char* pattern, const char fl)
 
 
 /* Load patterns/regexes from file */
-int load_regex_matcher(struct regex_matcher* matcher,FILE* fd,unsigned int options,int is_whitelist,struct cli_dbio *dbio)
+int load_regex_matcher(struct regex_matcher* matcher,FILE* fd,unsigned int *signo,unsigned int options,int is_whitelist,struct cli_dbio *dbio)
 {
-	int rc,line=0;
+	int rc,line=0,entry=0;
 	char buffer[FILEBUFF];
 
 	assert(matcher);
@@ -446,7 +494,7 @@ int load_regex_matcher(struct regex_matcher* matcher,FILE* fd,unsigned int optio
 		return CL_EMALFDB; /* already failed to load */
 	if(!fd && !dbio) {
 		cli_errmsg("Unable to load regex list (null file)\n");
-		return CL_EIO;
+		return CL_ENULLARG;
 	}
 
 	cli_dbgmsg("Loading regex_list\n");
@@ -484,13 +532,14 @@ int load_regex_matcher(struct regex_matcher* matcher,FILE* fd,unsigned int optio
 		size_t pattern_len;
 
 		cli_chomp(buffer);
+		line++;
 		if(!*buffer)
 			continue;/* skip empty lines */
 
 		if(functionality_level_check(buffer))
 			continue;
 
-		line++;
+		entry++;
 		pattern = strchr(buffer,':');
 		if(!pattern) {
 			cli_errmsg("Malformed regex list line %d\n",line);
@@ -521,17 +570,27 @@ int load_regex_matcher(struct regex_matcher* matcher,FILE* fd,unsigned int optio
 			/*matches displayed host*/
 			if (( rc = add_static_pattern(matcher, pattern) ))
 				return rc==CL_EMEM ? CL_EMEM : CL_EMALFDB;
-		} else if (buffer[0] == 'U' && !is_whitelist) {
+		} else if (buffer[0] == 'S' && (!is_whitelist || pattern[0]=='W')) {
 			pattern[pattern_len] = '\0';
-			if (( rc = add_hash(matcher, pattern, flags[0]) )) {
+			if (pattern[0]=='W')
+			    flags[0]='W';
+			if((pattern[0]=='W' || pattern[0]=='F' || pattern[0]=='P') && pattern[1]==':') {
+			    pattern += 2;
+			    if (( rc = add_hash(matcher, pattern, flags[0], pattern[-2] == 'P') )) {
 				cli_errmsg("Error loading at line: %d\n", line);
 				return rc==CL_EMEM ? CL_EMEM : CL_EMALFDB;
+			    }
+			} else {
+			    cli_errmsg("Error loading line: %d, %c\n", line, *pattern);
+			    return CL_EMALFDB;
 			}
 		} else {
 			return CL_EMALFDB;
 		}
 	}
 	matcher->list_loaded = 1;
+	if(signo)
+	    *signo += entry;
 
 	return CL_SUCCESS;
 }
@@ -548,10 +607,11 @@ int cli_build_regex_list(struct regex_matcher* matcher)
 		return -1;/*TODO: better error code */
 	}
 	cli_dbgmsg("Building regex list\n");
-	hashtab_free(&matcher->suffix_hash);
+	cli_hashtab_free(&matcher->suffix_hash);
 	if(( rc = cli_ac_buildtrie(&matcher->suffixes) ))
 		return rc;
 	matcher->list_built=1;
+	cli_hashset_destroy(&matcher->sha256_pfx_set);
 
 	return CL_SUCCESS;
 }
@@ -561,7 +621,7 @@ void regex_list_done(struct regex_matcher* matcher)
 {
 	assert(matcher);
 
-	if(matcher->list_inited) {
+	if(matcher->list_inited == 1) {
 		size_t i;
 		cli_ac_free(&matcher->suffixes);
 		if(matcher->suffix_regexes) {
@@ -581,12 +641,13 @@ void regex_list_done(struct regex_matcher* matcher)
 			for(i=0;i<matcher->regex_cnt;i++) {
 				regex_t *r = matcher->all_pregs[i];
 				cli_regfree(r);
-				free(r);
+				mpool_free(matcher->mempool, r);
 			}
-			free(matcher->all_pregs);
+			mpool_free(matcher->mempool, matcher->all_pregs);
 		}
-		hashtab_free(&matcher->suffix_hash);
-		cli_bm_free(&matcher->md5_hashes);
+		cli_hashtab_free(&matcher->suffix_hash);
+		cli_bm_free(&matcher->sha256_hashes);
+		cli_bm_free(&matcher->hostkey_prefix);
 	}
 }
 
@@ -599,7 +660,7 @@ int is_regex_ok(struct regex_matcher* matcher)
 static int add_newsuffix(struct regex_matcher *matcher, struct regex_list *info, const char *suffix, size_t len)
 {
 	struct cli_matcher *root = &matcher->suffixes;
-	struct cli_ac_patt *new = cli_calloc(1,sizeof(*new));
+	struct cli_ac_patt *new = mpool_calloc(matcher->mempool,1,sizeof(*new));
 	size_t i;
 	int ret;
 
@@ -621,9 +682,9 @@ static int add_newsuffix(struct regex_matcher *matcher, struct regex_list *info,
 	if(new->length > root->maxpatlen)
 		root->maxpatlen = new->length;
 
-	new->pattern = cli_malloc(sizeof(new->pattern[0])*len);
+	new->pattern = mpool_malloc(matcher->mempool, sizeof(new->pattern[0])*len);
 	if(!new->pattern) {
-		free(new);
+		mpool_free(matcher->mempool, new);
 		return CL_EMEM;
 	}
 	for(i=0;i<len;i++)
@@ -632,8 +693,8 @@ static int add_newsuffix(struct regex_matcher *matcher, struct regex_list *info,
 	new->customdata = info;
 	new->virname = NULL;
 	if((ret = cli_ac_addpatt(root,new))) {
-		free(new->pattern);
-		free(new);
+		mpool_free(matcher->mempool, new->pattern);
+		mpool_free(matcher->mempool, new);
 		return ret;
 	}
 	SO_preprocess_add(&matcher->filter, (const unsigned char*)suffix, len);
@@ -658,7 +719,7 @@ static int add_pattern_suffix(void *cbdata, const char *suffix, size_t suffix_le
 {
 	struct regex_matcher *matcher = cbdata;
 	struct regex_list *regex = cli_malloc(sizeof(*regex));
-	const struct element *el;
+	const struct cli_element *el;
 
 	assert(matcher);
 	if(!regex)
@@ -666,25 +727,25 @@ static int add_pattern_suffix(void *cbdata, const char *suffix, size_t suffix_le
 	regex->pattern = iregex->pattern ? cli_strdup(iregex->pattern) : NULL;
 	regex->preg = iregex->preg;
 	regex->nxt = NULL;
-	el = hashtab_find(&matcher->suffix_hash, suffix, suffix_len);
+	el = cli_hashtab_find(&matcher->suffix_hash, suffix, suffix_len);
 	/* TODO: what if suffixes are prefixes of eachother and only one will
 	 * match? */
 	if(el) {
 		/* existing suffix */
 		assert((size_t)el->data < matcher->suffix_cnt);
 		list_add_tail(&matcher->suffix_regexes[el->data], regex);
-		cli_dbgmsg(MODULE "added new regex to existing suffix %s: %s\n", suffix, regex->pattern);
 	} else {
 		/* new suffix */
 		size_t n = matcher->suffix_cnt++;
-		el = hashtab_insert(&matcher->suffix_hash, suffix, suffix_len, n);
+		el = cli_hashtab_insert(&matcher->suffix_hash, suffix, suffix_len, n);
 		matcher->suffix_regexes = cli_realloc(matcher->suffix_regexes, (n+1)*sizeof(*matcher->suffix_regexes));
 		if(!matcher->suffix_regexes)
 			return CL_EMEM;
 		matcher->suffix_regexes[n].tail = regex;
 		matcher->suffix_regexes[n].head = regex;
+		if (suffix[0] == '/' && suffix[1] == '\0')
+		    matcher->root_regex_idx = n;
 		add_newsuffix(matcher, regex, suffix, suffix_len);
-		cli_dbgmsg(MODULE "added new suffix %s, for regex: %s\n", suffix, regex->pattern);
 	}
 	return 0;
 }
@@ -704,10 +765,10 @@ static size_t reverse_string(char *pattern)
 static regex_t *new_preg(struct regex_matcher *matcher)
 {
 	regex_t *r;
-	matcher->all_pregs = cli_realloc(matcher->all_pregs, ++matcher->regex_cnt * sizeof(*matcher->all_pregs));
+	matcher->all_pregs = mpool_realloc(matcher->mempool, matcher->all_pregs, ++matcher->regex_cnt * sizeof(*matcher->all_pregs));
 	if(!matcher->all_pregs)
 		return NULL;
-	r = cli_malloc(sizeof(*r));
+	r = mpool_malloc(matcher->mempool, sizeof(*r));
 	if(!r)
 		return NULL;
 	matcher->all_pregs[matcher->regex_cnt-1] = r;
