@@ -1174,14 +1174,20 @@ cli_wm_decrypt_macro(int fd, off_t offset, uint32_t len, unsigned char key)
 static int
 skip_past_nul(int fd)
 {
-	char ch;
+    char *end;
+    char smallbuf[128];
 
-	do
-		if(cli_readn(fd, &ch, 1) != 1)
-			return FALSE;
-	while(ch);
-
-	return TRUE;
+    do {
+	int nread = cli_readn(fd, smallbuf, sizeof(smallbuf));
+	if (nread <= 0)
+	    return FALSE;
+	end = memchr(smallbuf, '\0', nread);
+	if (end) {
+	    if (lseek(fd, 1 + (end-smallbuf) - nread, SEEK_CUR) < 0)
+		return FALSE;
+	    return TRUE;
+	}
+    } while (1);
 }
 
 /*
