@@ -35,22 +35,24 @@
 #include "shared/misc.h"
 #include "shared/output.h"
 #include "shared/cdiff.h"
-#include "shared/sha256.h"
+#include "libclamav/sha256.h"
 
 #include "libclamav/str.h"
 #include "libclamav/others.h"
 #include "libclamav/cvd.h"
+#include "libclamav/default.h"
 
 #include "zlib.h"
 
-#ifdef HAVE_GMP
 #include "libclamav/dsig.h"
 
 #define PSS_NSTR "14783905874077467090262228516557917570254599638376203532031989214105552847269687489771975792123442185817287694951949800908791527542017115600501303394778618535864845235700041590056318230102449612217458549016089313306591388590790796515819654102320725712300822356348724011232654837503241736177907784198700834440681124727060540035754699658105895050096576226753008596881698828185652424901921668758326578462003247906470982092298106789657211905488986281078346361469524484829559560886227198091995498440676639639830463593211386055065360288422394053998134458623712540683294034953818412458362198117811990006021989844180721010947"
 #define PSS_ESTR "100002053"
 #define PSS_NBITS 2048
 #define PSS_DIGEST_LENGTH 32
-#endif /* HAVE_GMP */
+
+/* the line size can be changed from within .cdiff */
+#define CDIFF_LINE_SIZE CLI_DEFAULT_LSIG_BUFSIZE + 32
 
 struct cdiff_node {
     unsigned int lineno;
@@ -68,16 +70,16 @@ struct cdiff_ctx {
 struct cdiff_cmd {
     const char *name;
     unsigned short argc;
-    int (*handler)(const char *, struct cdiff_ctx *);
+    int (*handler)(const char *, struct cdiff_ctx *, char *, unsigned int);
 };
 
-static int cdiff_cmd_open(const char *cmdstr, struct cdiff_ctx *ctx);
-static int cdiff_cmd_add(const char *cmdstr, struct cdiff_ctx *ctx);
-static int cdiff_cmd_del(const char *cmdstr, struct cdiff_ctx *ctx);
-static int cdiff_cmd_xchg(const char *cmdstr, struct cdiff_ctx *ctx);
-static int cdiff_cmd_close(const char *cmdstr, struct cdiff_ctx *ctx);
-static int cdiff_cmd_move(const char *cmdstr, struct cdiff_ctx *ctx);
-static int cdiff_cmd_unlink(const char *cmdstr, struct cdiff_ctx *ctx);
+static int cdiff_cmd_open(const char *cmdstr, struct cdiff_ctx *ctx, char *lbuf, unsigned int lbuflen);
+static int cdiff_cmd_add(const char *cmdstr, struct cdiff_ctx *ctx, char *lbuf, unsigned int lbuflen);
+static int cdiff_cmd_del(const char *cmdstr, struct cdiff_ctx *ctx, char *lbuf, unsigned int lbuflen);
+static int cdiff_cmd_xchg(const char *cmdstr, struct cdiff_ctx *ctx, char *lbuf, unsigned int lbuflen);
+static int cdiff_cmd_close(const char *cmdstr, struct cdiff_ctx *ctx, char *lbuf, unsigned int lbuflen);
+static int cdiff_cmd_move(const char *cmdstr, struct cdiff_ctx *ctx, char *lbuf, unsigned int lbuflen);
+static int cdiff_cmd_unlink(const char *cmdstr, struct cdiff_ctx *ctx, char *lbuf, unsigned int lbuflen);
 
 static struct cdiff_cmd commands[] = {
     /* OPEN db_name */
@@ -171,7 +173,7 @@ static char *cdiff_token(const char *line, unsigned int token, unsigned int last
     return buffer;
 }
 
-static int cdiff_cmd_open(const char *cmdstr, struct cdiff_ctx *ctx)
+static int cdiff_cmd_open(const char *cmdstr, struct cdiff_ctx *ctx, char *lbuf, unsigned int lbuflen)
 {
 	char *db;
 	unsigned int i;
@@ -200,7 +202,7 @@ static int cdiff_cmd_open(const char *cmdstr, struct cdiff_ctx *ctx)
     return 0;
 }
 
-static int cdiff_cmd_add(const char *cmdstr, struct cdiff_ctx *ctx)
+static int cdiff_cmd_add(const char *cmdstr, struct cdiff_ctx *ctx, char *lbuf, unsigned int lbuflen)
 {
 	char *sig;
 	struct cdiff_node *new;
@@ -229,7 +231,7 @@ static int cdiff_cmd_add(const char *cmdstr, struct cdiff_ctx *ctx)
     return 0;
 }
 
-static int cdiff_cmd_del(const char *cmdstr, struct cdiff_ctx *ctx)
+static int cdiff_cmd_del(const char *cmdstr, struct cdiff_ctx *ctx, char *lbuf, unsigned int lbuflen)
 {
 	char *arg;
 	struct cdiff_node *pt, *last, *new;
@@ -286,7 +288,7 @@ static int cdiff_cmd_del(const char *cmdstr, struct cdiff_ctx *ctx)
     return 0;
 }
 
-static int cdiff_cmd_xchg(const char *cmdstr, struct cdiff_ctx *ctx)
+static int cdiff_cmd_xchg(const char *cmdstr, struct cdiff_ctx *ctx, char *lbuf, unsigned int lbuflen)
 {
 	char *arg, *arg2;
 	struct cdiff_node *pt, *last, *new;
@@ -351,11 +353,11 @@ static int cdiff_cmd_xchg(const char *cmdstr, struct cdiff_ctx *ctx)
     return 0;
 }
 
-static int cdiff_cmd_close(const char *cmdstr, struct cdiff_ctx *ctx)
+static int cdiff_cmd_close(const char *cmdstr, struct cdiff_ctx *ctx, char *lbuf, unsigned int lbuflen)
 {
 	struct cdiff_node *add, *del, *xchg;
 	unsigned int lines = 0;
-	char *tmp, line[1024];
+	char *tmp;
 	FILE *fh, *tmpfh;
 
 
@@ -388,11 +390,11 @@ static int cdiff_cmd_close(const char *cmdstr, struct cdiff_ctx *ctx)
 	    return -1;
 	}
 
-	while(fgets(line, sizeof(line), fh)) {
+	while(fgets(lbuf, lbuflen, fh)) {
 	    lines++;
 
 	    if(del && del->lineno == lines) {
-		if(strncmp(line, del->str, strlen(del->str))) {
+		if(strncmp(lbuf, del->str, strlen(del->str))) {
 		    fclose(fh);
 		    fclose(tmpfh);
 		    unlink(tmp);
@@ -400,13 +402,12 @@ static int cdiff_cmd_close(const char *cmdstr, struct cdiff_ctx *ctx)
 		    logg("!cdiff_cmd_close: Can't apply DEL at line %d of %s\n", lines, ctx->open_db);
 		    return -1;
 		}
-
 		del = del->next;
 		continue;
 	    }
 
 	    if(xchg && xchg->lineno == lines) {
-		if(strncmp(line, xchg->str, strlen(xchg->str))) {
+		if(strncmp(lbuf, xchg->str, strlen(xchg->str))) {
 		    fclose(fh);
 		    fclose(tmpfh);
 		    unlink(tmp);
@@ -427,7 +428,7 @@ static int cdiff_cmd_close(const char *cmdstr, struct cdiff_ctx *ctx)
 		continue;
 	    }
 
-	    if(fputs(line, tmpfh) == EOF) {
+	    if(fputs(lbuf, tmpfh) == EOF) {
 		fclose(fh);
 		fclose(tmpfh);
 		unlink(tmp);
@@ -488,10 +489,10 @@ static int cdiff_cmd_close(const char *cmdstr, struct cdiff_ctx *ctx)
     return 0;
 }
 
-static int cdiff_cmd_move(const char *cmdstr, struct cdiff_ctx *ctx)
+static int cdiff_cmd_move(const char *cmdstr, struct cdiff_ctx *ctx, char *lbuf, unsigned int lbuflen)
 {
 	unsigned int lines = 0, start_line, end_line;
-	char *arg, *srcdb, *dstdb, *tmpdb, line[1024], *start_str, *end_str;
+	char *arg, *srcdb, *dstdb, *tmpdb, *start_str, *end_str;
 	FILE *src, *dst, *tmp;
 
 
@@ -587,11 +588,11 @@ static int cdiff_cmd_move(const char *cmdstr, struct cdiff_ctx *ctx)
 	return -1;
     }
 
-    while(fgets(line, sizeof(line), src)) {
+    while(fgets(lbuf, lbuflen, src)) {
 	lines++;
 
 	if(lines == start_line) {
-	    if(strncmp(line, start_str, strlen(start_str))) {
+	    if(strncmp(lbuf, start_str, strlen(start_str))) {
 		free(start_str);
 		free(end_str);
 		free(srcdb);
@@ -606,7 +607,7 @@ static int cdiff_cmd_move(const char *cmdstr, struct cdiff_ctx *ctx)
 	    }
 
 	    do {
-		if(fputs(line, dst) == EOF) {
+		if(fputs(lbuf, dst) == EOF) {
 		    free(start_str);
 		    free(end_str);
 		    free(srcdb);
@@ -619,14 +620,14 @@ static int cdiff_cmd_move(const char *cmdstr, struct cdiff_ctx *ctx)
 		    free(dstdb);
 		    return -1;
 		}
-	    } while((lines < end_line) && fgets(line, sizeof(line), src) && lines++);
+	    } while((lines < end_line) && fgets(lbuf, lbuflen, src) && lines++);
 
 	    fclose(dst);
+	    dst = NULL;
 	    free(dstdb);
-	    dstdb = NULL;
 	    free(start_str);
 
-	    if(strncmp(line, end_str, strlen(end_str))) {
+	    if(strncmp(lbuf, end_str, strlen(end_str))) {
 		free(end_str);
 		free(srcdb);
 		fclose(src);
@@ -641,7 +642,13 @@ static int cdiff_cmd_move(const char *cmdstr, struct cdiff_ctx *ctx)
 	    continue;
 	}
 
-	if(fputs(line, tmp) == EOF) {
+	if(fputs(lbuf, tmp) == EOF) {
+	    if(dst) {
+		fclose(dst);
+		free(dstdb);
+		free(start_str);
+		free(end_str);
+	    }
 	    free(srcdb);
 	    fclose(src);
 	    fclose(tmp);
@@ -655,7 +662,7 @@ static int cdiff_cmd_move(const char *cmdstr, struct cdiff_ctx *ctx)
     fclose(src);
     fclose(tmp);
 
-    if(dstdb) {
+    if(dst) {
 	fclose(dst);
 	free(start_str);
 	free(end_str);
@@ -689,7 +696,7 @@ static int cdiff_cmd_move(const char *cmdstr, struct cdiff_ctx *ctx)
     return 0;
 }
 
-static int cdiff_cmd_unlink(const char *cmdstr, struct cdiff_ctx *ctx)
+static int cdiff_cmd_unlink(const char *cmdstr, struct cdiff_ctx *ctx, char *lbuf, unsigned int lbuflen)
 {
 	char *db;
 	unsigned int i;
@@ -723,10 +730,10 @@ static int cdiff_cmd_unlink(const char *cmdstr, struct cdiff_ctx *ctx)
     return 0;
 }
 
-static int cdiff_execute(const char *cmdstr, struct cdiff_ctx *ctx)
+static int cdiff_execute(const char *cmdstr, struct cdiff_ctx *ctx, char *lbuf, unsigned int lbuflen)
 {
 	char *cmd_name, *tmp;
-	int (*cmd_handler)(const char *, struct cdiff_ctx *) = NULL;
+	int (*cmd_handler)(const char *, struct cdiff_ctx *, char *, unsigned int) = NULL;
 	unsigned int i;
 
 
@@ -756,7 +763,7 @@ static int cdiff_execute(const char *cmdstr, struct cdiff_ctx *ctx)
     }
     free(tmp);
 
-    if(cmd_handler(cmdstr, ctx)) {
+    if(cmd_handler(cmdstr, ctx, lbuf, lbuflen)) {
 	logg("!cdiff_apply: Can't execute command %s\n", cmd_name);
 	free(cmd_name);
 	return -1;
@@ -766,7 +773,6 @@ static int cdiff_execute(const char *cmdstr, struct cdiff_ctx *ctx)
     return 0;
 }
 
-#ifdef HAVE_GMP
 static void pss_mgf(unsigned char *in, unsigned int inlen, unsigned char *out, unsigned int outlen)
 {
 	SHA256_CTX ctx;
@@ -785,8 +791,7 @@ static void pss_mgf(unsigned char *in, unsigned int inlen, unsigned char *out, u
 	sha256_init(&ctx);
 	sha256_update(&ctx, in, inlen);
 	sha256_update(&ctx, cnt, sizeof(cnt));
-	sha256_final(&ctx);
-	sha256_digest(&ctx, digest);
+	sha256_final(&ctx, digest);
 
 	if(i != laps - 1)
 	    memcpy(&out[i * PSS_DIGEST_LENGTH], digest, PSS_DIGEST_LENGTH);
@@ -797,7 +802,7 @@ static void pss_mgf(unsigned char *in, unsigned int inlen, unsigned char *out, u
 
 static int pss_versig(const unsigned char *sha256, const char *dsig)
 {
-	mpz_t n, e;
+	mp_int n, e;
 	SHA256_CTX ctx;
 	unsigned char *pt, digest1[PSS_DIGEST_LENGTH], digest2[PSS_DIGEST_LENGTH], *salt;
 	unsigned int plen = PSS_NBITS / 8, hlen, slen, i;
@@ -807,17 +812,17 @@ static int pss_versig(const unsigned char *sha256, const char *dsig)
 
 
     hlen = slen = PSS_DIGEST_LENGTH;
-    mpz_init_set_str(n, PSS_NSTR, 10);
-    mpz_init_set_str(e, PSS_ESTR, 10);
-
+    mp_init(&n);
+    mp_read_radix(&n, PSS_NSTR, 10);
+    mp_init(&e);
+    mp_read_radix(&e, PSS_ESTR, 10);
     if(!(pt = cli_decodesig(dsig, plen, e, n))) {
-	mpz_clear(n);
-	mpz_clear(e);
+	mp_clear(&n);
+	mp_clear(&e);
 	return -1;
     }
-
-    mpz_clear(n);
-    mpz_clear(e);
+    mp_clear(&n);
+    mp_clear(&e);
 
     if(pt[plen - 1] != 0xbc) {
 	/* cli_dbgmsg("cli_versigpss: Incorrect signature syntax (0xbc)\n"); */
@@ -854,8 +859,7 @@ static int pss_versig(const unsigned char *sha256, const char *dsig)
 
     sha256_init(&ctx);
     sha256_update(&ctx, fblock, sizeof(fblock));
-    sha256_final(&ctx);
-    sha256_digest(&ctx, digest1);
+    sha256_final(&ctx, digest1);
 
     if(memcmp(digest1, digest2, hlen)) {
 	/* cli_dbgmsg("cli_versigpss: Signature doesn't match.\n"); */
@@ -864,24 +868,21 @@ static int pss_versig(const unsigned char *sha256, const char *dsig)
 
     return 0;
 }
-#endif /* HAVE_GMP */
 
 int cdiff_apply(int fd, unsigned short mode)
 {
 	struct cdiff_ctx ctx;
 	FILE *fh;
 	gzFile *gzh;
-	char line[1024], buff[FILEBUFF], *dsig = NULL;
+	char *line, *lbuf, buff[FILEBUFF], *dsig = NULL;
 	unsigned int lines = 0, cmds = 0;
-	unsigned int difflen, diffremain;
+	unsigned int difflen, diffremain, line_size = CDIFF_LINE_SIZE;
 	int end, i, n;
 	struct stat sb;
 	int desc;
-#ifdef HAVE_GMP
 	SHA256_CTX sha256ctx;
 	unsigned char digest[32];
 	int sum, bread;
-#endif
 #define DSIGBUFF 350
 
     memset(&ctx, 0, sizeof(ctx));
@@ -891,18 +892,35 @@ int cdiff_apply(int fd, unsigned short mode)
 	return -1;
     }
 
+    if(!(line = malloc(line_size))) {
+	logg("!cdiff_apply: Can't allocate memory for 'line'\n");
+	close(desc);
+	return -1;
+    }
+
+    if(!(lbuf = malloc(line_size))) {
+	logg("!cdiff_apply: Can't allocate memory for 'lbuf'\n");
+	close(desc);
+	free(line);
+	return -1;
+    }
+
     if(mode == 1) { /* .cdiff */
 
 	if(lseek(desc, -DSIGBUFF, SEEK_END) == -1) {
 	    logg("!cdiff_apply: lseek(desc, %d, SEEK_END) failed\n", -DSIGBUFF);
 	    close(desc);
+	    free(line);
+	    free(lbuf);
 	    return -1;
 	}
 
-	memset(line, 0, sizeof(line));
+	memset(line, 0, line_size);
 	if(read(desc, line, DSIGBUFF) != DSIGBUFF) {
 	    logg("!cdiff_apply: Can't read %d bytes\n", DSIGBUFF);
 	    close(desc);
+	    free(line);
+	    free(lbuf);
 	    return -1;
 	}
 
@@ -916,12 +934,16 @@ int cdiff_apply(int fd, unsigned short mode)
 	if(!dsig) {
 	    logg("!cdiff_apply: No digital signature in cdiff file\n");
 	    close(desc);
+	    free(line);
+	    free(lbuf);
 	    return -1;
 	}
 
 	if(fstat(desc, &sb) == -1) {
 	    logg("!cdiff_apply: Can't fstat file\n");
 	    close(desc);
+	    free(line);
+	    free(lbuf);
 	    return -1;
 	}
 
@@ -929,16 +951,19 @@ int cdiff_apply(int fd, unsigned short mode)
 	if(end < 0) {
 	    logg("!cdiff_apply: compressed data end offset < 0\n");
 	    close(desc);
+	    free(line);
+	    free(lbuf);
 	    return -1;
 	}
 
 	if(lseek(desc, 0, SEEK_SET) == -1) {
 	    logg("!cdiff_apply: lseek(desc, 0, SEEK_SET) failed\n");
 	    close(desc);
+	    free(line);
+	    free(lbuf);
 	    return -1;
 	}
 
-#ifdef HAVE_GMP
 	sha256_init(&sha256ctx);
 	sum = 0;
 	while((bread = read(desc, buff, FILEBUFF)) > 0) {
@@ -950,19 +975,21 @@ int cdiff_apply(int fd, unsigned short mode)
 	    }
 	    sum += bread;
 	}
-	sha256_final(&sha256ctx);
-	sha256_digest(&sha256ctx, digest);
+	sha256_final(&sha256ctx, digest);
 
 	if(pss_versig(digest, dsig)) {
 	    logg("!cdiff_apply: Incorrect digital signature\n");
 	    close(desc);
+	    free(line);
+	    free(lbuf);
 	    return -1;
 	}
-#endif
 
 	if(lseek(desc, 0, SEEK_SET) == -1) {
 	    logg("!cdiff_apply: lseek(desc, 0, SEEK_SET) failed\n");
 	    close(desc);
+	    free(line);
+	    free(lbuf);
 	    return -1;
 	}
 
@@ -978,36 +1005,67 @@ int cdiff_apply(int fd, unsigned short mode)
 	if(sscanf(buff, "ClamAV-Diff:%*u:%u:", &difflen) != 1) {
 	    logg("!cdiff_apply: Incorrect file format\n");
 	    close(desc);
+	    free(line);
+	    free(lbuf);
 	    return -1;
 	}
 
 	if(!(gzh = gzdopen(desc, "rb"))) {
 	    logg("!cdiff_apply: Can't gzdopen descriptor %d\n", desc);
 	    close(desc);
+	    free(line);
+	    free(lbuf);
 	    return -1;
 	}
 
 	diffremain = difflen;
 	while(diffremain) {
-	    unsigned int bufsize = diffremain < sizeof(line) ? diffremain + 1 : sizeof(line);
+	    unsigned int bufsize = diffremain < line_size ? diffremain + 1 : line_size;
 
 	    if(!gzgets(gzh, line, bufsize)) {
 		logg("!cdiff_apply: Premature EOF at line %d\n", lines + 1);
 		cdiff_ctx_free(&ctx);
 		gzclose(gzh);
+		free(line);
+		free(lbuf);
 		return -1;
 	    }
 	    diffremain -= strlen(line);
 	    lines++;
 	    cli_chomp(line);
 
-	    if(line[0] == '#' || !strlen(line))
+	    if(!strlen(line))
 		continue;
+	    if(line[0] == '#') {
+		if(!strncmp(line, "#LSIZE", 6) && sscanf(line, "#LSIZE %u", &line_size) == 1) {
+			char *r1, *r2;
+		    if(line_size < CDIFF_LINE_SIZE || line_size > 10485760) {
+			logg("^cdiff_apply: Ignoring new buffer size request - invalid size %d\n", line_size);
+			line_size = CDIFF_LINE_SIZE;
+			continue;
+		    }
+		    r1 = realloc(line, line_size);
+		    r2 = realloc(lbuf, line_size);
+		    if(!r1 || !r2) {
+			logg("!cdiff_apply: Can't resize line buffer to %d bytes\n", line_size);
+			cdiff_ctx_free(&ctx);
+			gzclose(gzh);
+			free(line);
+			free(lbuf);
+			return -1;
+		    }
+		    line = r1;
+		    lbuf = r2;
+		}
+		continue;
+	    }
 
-	    if(cdiff_execute(line, &ctx) == -1) {
+	    if(cdiff_execute(line, &ctx, lbuf, line_size) == -1) {
 		logg("!cdiff_apply: Error executing command at line %d\n", lines);
 		cdiff_ctx_free(&ctx);
 		gzclose(gzh);
+		free(line);
+		free(lbuf);
 		return -1;
 	    } else {
 		cmds++;
@@ -1020,20 +1078,47 @@ int cdiff_apply(int fd, unsigned short mode)
 	if(!(fh = fdopen(desc, "r"))) {
 	    logg("!cdiff_apply: fdopen() failed for descriptor %d\n", desc);
 	    close(desc);
+	    free(line);
+	    free(lbuf);
 	    return -1;
 	}
 
-	while(fgets(line, sizeof(line), fh)) {
+	while(fgets(line, line_size, fh)) {
 	    lines++;
 	    cli_chomp(line);
 
-	    if(line[0] == '#' || !strlen(line))
+	    if(!strlen(line))
 		continue;
+	    if(line[0] == '#') {
+		if(!strncmp(line, "#LSIZE", 6) && sscanf(line, "#LSIZE %u", &line_size) == 1) {
+			char *r1, *r2;
+		    if(line_size < CDIFF_LINE_SIZE || line_size > 10485760) {
+			logg("^cdiff_apply: Ignoring new buffer size request - invalid size %d\n", line_size);
+			line_size = CDIFF_LINE_SIZE;
+			continue;
+		    }
+		    r1 = realloc(line, line_size);
+		    r2 = realloc(lbuf, line_size);
+		    if(!r1 || !r2) {
+			logg("!cdiff_apply: Can't resize line buffer to %d bytes\n", line_size);
+			cdiff_ctx_free(&ctx);
+			fclose(fh);
+			free(line);
+			free(lbuf);
+			return -1;
+		    }
+		    line = r1;
+		    lbuf = r2;
+		}
+		continue;
+	    }
 
-	    if(cdiff_execute(line, &ctx) == -1) {
+	    if(cdiff_execute(line, &ctx, lbuf, line_size) == -1) {
 		logg("!cdiff_apply: Error executing command at line %d\n", lines);
 		cdiff_ctx_free(&ctx);
 		fclose(fh);
+		free(line);
+		free(lbuf);
 		return -1;
 	    } else {
 		cmds++;
@@ -1043,6 +1128,9 @@ int cdiff_apply(int fd, unsigned short mode)
 	fclose(fh);
     }
 
+    free(line);
+    free(lbuf);
+
     if(ctx.open_db) {
 	logg("*cdiff_apply: File %s was not properly closed\n", ctx.open_db);
 	cdiff_ctx_free(&ctx);
