@@ -998,13 +998,22 @@ static int sigtool_scandir (const char *dirname, int hex_output)
 
 				/* generate the temporary directory */
 				dir = cli_gentemp (tmpdir);
+				if(!dir) {
+				    printf("cli_gentemp() failed\n");
+				    closedir (dd);
+				    return -1;
+				}
 				if (mkdir (dir, 0700)) {
 				    printf ("Can't create temporary directory %s\n", dir);
+				    closedir (dd);
+				    free(dir);
 				    return CL_ETMPDIR;
 				}
 
 				if ((desc = open (fname, O_RDONLY)) == -1) {
 				    printf ("Can't open file %s\n", fname);
+				    closedir (dd);
+				    free(dir);
 				    return 1;
 				}
 
@@ -1012,8 +1021,11 @@ static int sigtool_scandir (const char *dirname, int hex_output)
 				    printf ("ERROR %s\n", cl_strerror (ret));
 				    cli_rmdirs (dir);
 				    free (dir);
+				    close(desc);
+				    closedir (dd);
 				    return ret;
 				}
+				close(desc);
 
 				if(vba)
 				    sigtool_vba_scandir (dir, hex_output, vba);
