@@ -31,6 +31,7 @@ static char *dir;
 
 static void htmlnorm_setup(void)
 {
+        cl_init(CL_INIT_DEFAULT);
 	dconf_setup();
 	dir = cli_gentemp(NULL);
 	fail_unless(!!dir, "cli_gentemp failed");
@@ -39,7 +40,8 @@ static void htmlnorm_setup(void)
 static void htmlnorm_teardown(void)
 {
 	dconf_teardown();
-	fail_unless(cli_rmdirs(dir) == 0, "rmdirs failed");
+	/* can't call fail() functions in teardown, it can cause SEGV */
+	cli_rmdirs(dir);
 	free(dir);
 	dir = NULL;
 }
